/* indent.c - Adjust indentation of file - 1.2 */

/*
** This filter will adjust the indentation of an existing file.  It
** is most useful for C source created with an editor program that 
** doesn't agree with your notion of tab stop locations.  I use it to
** make the output of cb(1) have a nesting level of 4, rather than 8.
** In the spirit of Unix tools, I'm sure you will find other uses.
*/

/*
** Author:
**   Chad R. Larson			This program is placed in the
**   DCF, Inc.				Public Domain.  You may do with
**   14623 North 49th Place		it as you please.
**   Scottsdale, AZ 85254
*/

/*
** Usage is "indent [-tn] [-rn] [-en] [-s] [file]"
**   Where:
**	-tn	Set hardware tab stop (for both input and output)
**		to n.  If this option is omitted, n=8.
**	-rn	Set reduction factor to n.  That is, if n=3 each
**		output line will be indented 1/3 as much as its
**		corresponding input line.  Default value for n is 2.
**	-en	Set expansion factor to n.  That is, if n=3 each
**		output line will be indented 3 times as much as its
**		corresponding input line.  Default value for n is 1.
**		The -e and -r options may be used in conjunction,
**		for example to get a two thirds indentation reduction.
**	-s	Supress tabs in output.  Normally, multiple leading
**		spaces will be replaced with the necessary amount of
**		tab characters to reduce the size of the output file.
**		Note this option only applies to leading white space.
**		No other tabs will be affected.
**	If no output file is specified, the standard input will be read.
**	Output is to the standard output.  Error messages will be printed
**	on standard error (pretty standard, hmmm?).
*/

#include <stdio.h>

#define	FALSE 0
#define TRUE ~FALSE
#define MAX_LINE 512

/* linked in later */
void	exit();

/* give the operator a clue */
void usage(prog_name)
char	*prog_name;
{
    (void) fprintf(stderr, "%s version 1.2 - 9/20/88\n", prog_name);
    (void) fprintf(stderr,
      "Usage: %s [-tn] [-rn] [-en] [-s] [file]\n", prog_name);
    exit(1);
}

/* here it is */
void main(argc, argv)
int	argc;
char	*argv[];
{
    int			expand = 1;		/* expansion factor */
    int			reduce = 2;		/* reduction factor */
    int			tabsize = 8;		/* hardware tab size */
    int			tabflag = TRUE;		/* insert tabs in output? */
    int			c;			/* generic character */
    int			i;			/* generic integer */
    extern int		optind;			/* argument index */
    extern char		*optarg;		/* argument to options */
    FILE		*infile;		/* input file descriptor */
    char		inbuf[MAX_LINE];	/* input buffer */
    char		outbuf[MAX_LINE];	/* output buffer */
    register char	*ip, *op;		/* buffer in & out pointers */
    register int	count;			/* whitespace counter */

    char *progname = NULL;
    char *basename();

	progname = basename(argv[0]);

    /* sort out the command line options, if any */
    while ((c = getopt(argc, argv, "st:r:e:")) != EOF)
	switch (c) {
	case 's':
	    tabflag = FALSE;
	    break;
	case 't':
	    tabsize = atoi(optarg);
	    break;
	case 'r':
	    reduce = atoi(optarg);
	    break;
	case 'e':
	    expand = atoi(optarg);
	    break;
	case '?':
	    usage(progname);
	}

    /* open input file or stdin as required */
    if (argv[optind] != NULL) {
	if ((infile = fopen(argv[optind], "r")) == NULL) {
	    (void) fprintf(stderr,
	      "%s: Cannot open %s\n", progname, argv[optind]);
	    exit(2);
	}
    } else {
	if (isatty(fileno(stdin)) == 0)
	    infile = stdin;
	else
	    usage(progname);
    }

    /* options are parsed, input file is open, let's go! */
    while ((ip = fgets(inbuf, MAX_LINE, infile)) != NULL) {

	/* count leading white space */
	count = 0;
	while ((c = *ip++) == ' ' || c == '\t')
	    switch (c) {
	    case ' ':
		count++;
		break;
	    case '\t':
		count += tabsize - (count % tabsize);
		break;
	    }
	    ip--;

	/* ratio the white space */
	count = (count * expand) / reduce;

	/* put proper white space in output buffer */
	op = outbuf;
	if (tabflag) {
	    i = count / tabsize;
	    while (i--)
		*op++ = '\t';
	    count = count % tabsize;
	}
	while (count--)
	    *op++ = ' ';

	/* copy rest of input to output */
	while (*op++ = *ip++);

	/* dump the output buffer */
	(void) fputs(outbuf, stdout);
    }
    exit(0);
}

char *basename(s)
char *s;
{
	char *p;
	if (p = strrchr(s, '.'))
		*p = '\0';
	if (p = strrchr(s, '\\'))
		return(++p);
	else
		return(s);
}  /* basename */



